## Added by Brian Blaylock
## June 21, 2023

"""
A Herbie template for the High Resolution Deterministic Prediction System (HRDPS)

Meteorological Service of Canada (MSC)
The HRDPS is Canada's 2.5 km deterministic model

Description: https://eccc-msc.github.io/open-data/msc-data/nwp_hrdps/readme_hrdps-datamart_en/#data-location
Data Source: https://dd.weather.gc.ca/model_hrdps/

- Continental domain: https://dd.weather.gc.ca/model_hrdps/continental/{res}/{HH}/{hhh}/
- North domain (experimental): https://dd.weather.gc.ca/model_hrdps/north/grib2/{HH}/{hhh}/

where `HH` is the models initialization time and `hhh` is the forecast lead time.

Data available for last 24 hours.

Other Levels
------------
`variable` is one of {'ALBDO', 'CAPE', 'CWAT', 'DEN', 'DEPR', 'DLWRF', 'DPT', 'DSWRF',
       'GUST-Max', 'GUST-Min', 'GUST', 'HGT', 'HLCY', 'HPBL', 'ICEC',
       'LAND', 'LHTFL', 'NLWRS', 'NSWRS', 'PRES', 'PRMSL', 'RH', 'SDEN',
       'SDWE', 'SFCWRO', 'SHTFL', 'SKINT', 'SNOD', 'SOILVIC', 'SOILW',
       'SPFH', 'TCDC', 'TMP', 'TSOIL', 'UGRD', 'ULWRF', 'USWRF', 'UTCI',
       'VGRD', 'WDIR', 'WIND', 'WTMP'}
`level` is one of {'Sfc', 'EATM', 'AGL-120m', 'AGL-40m', 'AGL-80m', 'AGL-2m', 'NTAT',
       'AGL-10m', 'ISBY-1000-500', 'MSL', 'DBS-0-10cm', 'DBS-0-1cm'}

Isobaric Levels
---------------
`variable` is one of {'ABSV', 'DEPR', 'HGT', 'MU-VT-LI', 'RH', 'SHWINX', 'SPFH', 'TMP',
       'UGRD', 'VGRD', 'VVEL', 'WDIR', 'WIND'}
`level` is "ISBL_####" where #### is one of
    {'0250', '0500', '0700', '0850', '1000', '0050', '0100', '0150',
    '0175', '0200', '0225', '0275', '0300', '0350', '0400', '0450',
    '0550', '0600', '0650', '0750', '0800', '0875', '0900', '0925',
'0950', '0970', '0985', '1015'}
"""

_variable = {
    "ALBDO",
    "CAPE",
    "CWAT",
    "DEN",
    "DEPR",
    "DLWRF",
    "DPT",
    "DSWRF",
    "GUST-Max",
    "GUST-Min",
    "GUST",
    "HGT",
    "HLCY",
    "HPBL",
    "ICEC",
    "LAND",
    "LHTFL",
    "NLWRS",
    "NSWRS",
    "PRES",
    "PRMSL",
    "RH",
    "SDEN",
    "SDWE",
    "SFCWRO",
    "SHTFL",
    "SKINT",
    "SNOD",
    "SOILVIC",
    "SOILW",
    "SPFH",
    "TCDC",
    "TMP",
    "TSOIL",
    "UGRD",
    "ULWRF",
    "USWRF",
    "UTCI",
    "VGRD",
    "WDIR",
    "WIND",
    "WTMP",
}

_level = {
    "Sfc",
    "EATM",
    "AGL-120m",
    "AGL-40m",
    "AGL-80m",
    "AGL-2m",
    "NTAT",
    "AGL-10m",
    "ISBY-1000-500",
    "MSL",
    "DBS-0-10cm",
    "DBS-0-1cm",
    "ISBL_0250",
    "ISBL_0500",
    "ISBL_0700",
    "ISBL_0850",
    "ISBL_1000",
    "ISBL_0050",
    "ISBL_0100",
    "ISBL_0150",
    "ISBL_0175",
    "ISBL_0200",
    "ISBL_0225",
    "ISBL_0275",
    "ISBL_0300",
    "ISBL_0350",
    "ISBL_0400",
    "ISBL_0450",
    "ISBL_0550",
    "ISBL_0600",
    "ISBL_0650",
    "ISBL_0750",
    "ISBL_0800",
    "ISBL_0875",
    "ISBL_0900",
    "ISBL_0925",
    "ISBL_0950",
    "ISBL_0970",
    "ISBL_0985",
    "ISBL_1015",
}


class hrdps:
    def template(self):
        if not hasattr(self, "variable"):
            print(
                f"HRDPS requires an argument for 'variable'. Here are some ideas:\n{_variable}."
            )
            print(
                "For full list of files, see https://dd.weather.gc.ca/model_hrdps/continental/"
            )
        if not hasattr(self, "level"):
            print(
                f"HRDPS requires an argument for 'level'. Here are some ideas:\n{_level}"
            )
            print(
                "For full list of files, see https://dd.weather.gc.ca/model_hrdps/continental/"
            )

        self.DESCRIPTION = (
            "Canada's High Resolution Deterministic Prediction System (HRDPS)"
        )
        self.DETAILS = {
            "Datamart product description": "https://eccc-msc.github.io/open-data/msc-data/nwp_hrdps/readme_hrdps-datamart_en/#data-location",
        }
        self.PRODUCTS = {
            "continental/2.5km": "continental domain",
        }
        PATH = f"{self.date:%H}/{self.fxx:03d}/{self.date:%Y%m%dT%HZ}_MSC_HRDPS_{self.variable}_{self.level}_RLatLon0.0225_PT{self.fxx:03d}H.grib2"
        self.SOURCES = {
            "msc": f"https://dd.weather.gc.ca/model_hrdps/{self.product}/{PATH}",
            "msc2": f"https://dd.weather.gc.ca/model_hrdps/{self.product}/{PATH.replace('_HRDPS_', '_HRDPS-WEonG_')}",
        }

        self.IDX_SUFFIX = [".grb2.idx", ".idx", ".grib.idx"]
        self.LOCALFILE = f"{self.get_remoteFileName}"


class hrdps_north:
    def template(self):
        if not hasattr(self, "variable"):
            print(
                f"HRDPS requires an argument for 'variable'. Here are some ideas:\n{_variable}."
            )
            print(
                "For full list of files, see https://dd.weather.gc.ca/model_hrdps/north"
            )
        if not hasattr(self, "level"):
            print(
                f"HRDPS requires an argument for 'level'. Here are some ideas:\n{_level}"
            )
            print("For full list of files, see https://dd.weather.gc.ca/model_hrdps")

        self.DESCRIPTION = "Canada's High Resolution Deterministic Prediction System (HRDPS) North domain (experimental)"
        self.DETAILS = {
            "Datamart product description": "https://eccc-msc.github.io/open-data/msc-data/nwp_hrdps/readme_hrdps-datamart_en",
        }
        self.PRODUCTS = {
            "north/grib2": "North domain (experimental)",
        }
        PATH = f"{self.date:%H}/{self.fxx:03d}/CMC_hrdps_north_{self.variable}_{self.level}_ps2.5km_{self.date:%Y%m%d%H}_P{self.fxx:03d}-00.grib2"
        self.SOURCES = {
            "msc": f"https://dd.weather.gc.ca/model_hrdps/{self.product}/{PATH}",
        }

        self.IDX_SUFFIX = [".grb2.idx", ".idx", ".grib.idx"]
        self.LOCALFILE = f"{self.get_remoteFileName}"



## Added by Brian Blaylock
## July 26, 2021


class gdas:
    def template(self):
        self.DESCRIPTION = "Global Data Assimilation System"
        self.DETAILS = {
            "nomads product description": "https://www.nco.ncep.noaa.gov/pmb/products/gfs/#GDAS",
            "google cloud platform": "https://console.cloud.google.com/marketplace/product/noaa-public/gfs?q=search&referrer=search&project=python-232920",
            "azure document": "https://github.com/microsoft/AIforEarthDatasets#noaa-global-forecast-system-gfs",
            "aws document": "https://registry.opendata.aws/noaa-gfs-bdp-pds",
        }
        self.PRODUCTS = {
            "pgrb2.0p25": "common fields, 0.25 degree resolution",
            "pgrb2.1p00": "common fields, 1.00 degree resolution",
        }
        self.SOURCES = {
            "aws": f"https://noaa-gfs-bdp-pds.s3.amazonaws.com/gdas.{self.date:%Y%m%d/%H}/atmos/gdas.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
            "aws-old": f"https://noaa-gfs-bdp-pds.s3.amazonaws.com/gdas.{self.date:%Y%m%d/%H}/gdas.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
            "ftpprd": f"https://ftpprd.ncep.noaa.gov/data/nccf/com/gfs/prod/gdas.{self.date:%Y%m%d/%H}/atmos/gdas.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
            "nomads": f"https://nomads.ncep.noaa.gov/pub/data/nccf/com/gfs/prod/gdas.{self.date:%Y%m%d/%H}/atmos/gdas.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
            "google": f"https://storage.googleapis.com/global-forecast-system/gdas.{self.date:%Y%m%d/%H}/atmos/gdas.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
            "azure": f"https://noaagfs.blob.core.windows.net/gfs/gdas.{self.date:%Y%m%d/%H}/atmos/gfs.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
        }
        self.IDX_SUFFIX = [".idx"]
        self.LOCALFILE = f"{self.get_remoteFileName}"


class gfs:
    def template(self):
        self.DESCRIPTION = "Global Forecast System"
        self.DETAILS = {
            "nomads product description": "https://www.nco.ncep.noaa.gov/pmb/products/gfs",
            "google cloud platform": "https://console.cloud.google.com/marketplace/product/noaa-public/gfs?q=search&referrer=search&project=python-232920",
            "azure document": "https://github.com/microsoft/AIforEarthDatasets#noaa-global-forecast-system-gfs",
            "aws document": "https://registry.opendata.aws/noaa-gfs-bdp-pds",
        }
        self.PRODUCTS = {
            "pgrb2.0p25": "common fields, 0.25 degree resolution",
            "pgrb2.0p50": "common fields, 0.50 degree resolution",
            "pgrb2.1p00": "common fields, 1.00 degree resolution",
            "pgrb2b.0p25": "uncommon fields, 0.25 degree resolution",
            "pgrb2b.0p50": "uncommon fields, 0.50 degree resolution",
            "pgrb2b.1p00": "uncommon fields, 1.00 degree resolution",
            "pgrb2full.0p50": "combined grids of 0.50 resolution",
        }
        self.SOURCES = {
            "aws": f"https://noaa-gfs-bdp-pds.s3.amazonaws.com/gfs.{self.date:%Y%m%d/%H}/atmos/gfs.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
            "aws-old": f"https://noaa-gfs-bdp-pds.s3.amazonaws.com/gfs.{self.date:%Y%m%d/%H}/gfs.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
            "ftpprd": f"https://ftpprd.ncep.noaa.gov/data/nccf/com/gfs/prod/gfs.{self.date:%Y%m%d/%H}/atmos/gfs.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
            "nomads": f"https://nomads.ncep.noaa.gov/pub/data/nccf/com/gfs/prod/gfs.{self.date:%Y%m%d/%H}/atmos/gfs.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
            "google": f"https://storage.googleapis.com/global-forecast-system/gfs.{self.date:%Y%m%d/%H}/atmos/gfs.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
            "azure": f"https://noaagfs.blob.core.windows.net/gfs/gfs.{self.date:%Y%m%d/%H}/atmos/gfs.t{self.date:%H}z.{self.product}.f{self.fxx:03d}",
        }
        self.IDX_SUFFIX = [".idx"]
        self.LOCALFILE = f"{self.get_remoteFileName}"


class gfs_wave:
    def template(self):
        self.DESCRIPTION = "Global Forecast System - Wave Products"
        self.DETAILS = {
            "nomads product description": "https://www.nco.ncep.noaa.gov/pmb/products/gfs/#GFSwave",
        }
        self.PRODUCTS = {
            "arctic.9km": "Arctic; 9-km resolution",
            "atlocn.0p16": "North Atlantic 0.16 deg resolution",
            "epacif.0p16": "Eastern Pacific; .16 deg resolution",
            "global.0p16": "Global; 0.16 deg resolution",
            "global.0p25": "Global; 0.25 deg resolution",
            "gsouth.0p25": "Global South; 0.25 deg resolution",
            "wcoast.0p16": "West Coast; 0.16 deg resolution",
        }
        self.SOURCES = {
            "aws": f"https://noaa-gfs-bdp-pds.s3.amazonaws.com/gfs.{self.date:%Y%m%d/%H}/wave/gridded/gfswave.t{self.date:%H}z.{self.product}.f{self.fxx:03d}.grib2",
            "ftpprd": f"https://ftpprd.ncep.noaa.gov/data/nccf/com/gfs/prod/gfs.{self.date:%Y%m%d/%H}/wave/gridder/gfswave.t{self.date:%H}z.{self.product}.f{self.fxx:03d}.grib2",
            "nomads": f"https://nomads.ncep.noaa.gov/pub/data/nccf/com/gfs/prod/gfs.{self.date:%Y%m%d/%H}/wave/gridded/gfswave.t{self.date:%H}z.{self.product}.f{self.fxx:03d}.grib2",
            "google": f"https://storage.googleapis.com/global-forecast-system/gfs.{self.date:%Y%m%d/%H}/wave/gridded/gfswave.t{self.date:%H}z.{self.product}.f{self.fxx:03d}.grib2",
            "azure": f"https://noaahrrr.blob.core.windows.net/gfs/gfs.{self.date:%Y%m%d/%H}/wave/gridded/gfswave.t{self.date:%H}z.{self.product}.f{self.fxx:03d}.grib2",
        }
        self.LOCALFILE = f"{self.get_remoteFileName}"



## Added by Brian Blaylock
## January 3, 2023

"""
A Herbie template for the RTMA (Real-time Mesoscale Analysis).

Look at the urma.py file for the URMA analysis template

TODO: Precipitation products need help!

TODO: Clean up to look like URMA's template file.

TODO: Add all other domains.

"""


class rtma:
    def template(self):
        self.DESCRIPTION = "CONUS Real-Time Mesoscale Analysis (RTMA)"
        self.DETAILS = {
            "NOMADS product description": "https://www.nco.ncep.noaa.gov/pmb/products/rtma/",
            "Amazon Open Data": "https://registry.opendata.aws/noaa-rtma/",
        }
        self.PRODUCTS = {
            "anl": "Analysis",
            "err": "Analysis Forecast Error",
            "ges": "Forecasts",
            "pcp": "Precipitation Field",
        }

        if self.product != "pcp":
            PATH = f"rtma2p5.{self.date:%Y%m%d}/rtma2p5.t{self.date:%H}z.2dvar{self.product}_ndfd.grb2_wexp"
            self.SOURCES = {
                "aws": f"https://noaa-rtma-pds.s3.amazonaws.com/{PATH}",
                "nomads": f"https://nomads.ncep.noaa.gov/pub/data/nccf/com/rtma/prod/{PATH}",
            }
        else:
            PATH = f"rtma2p5.{self.date:%Y%m%d}/rtma2p5.{self.date:%Y%m%d%H}.{self.product}.184.grb2"
            self.SOURCES = {
                "aws": f"https://noaa-rtma-pds.s3.amazonaws.com/{PATH}",
                "nomads": f"https://nomads.ncep.noaa.gov/pub/data/nccf/com/rtma/prod/{PATH}",
            }

        self.IDX_SUFFIX = [".grb2.idx", ".idx", ".grib.idx"]
        self.LOCALFILE = f"{self.get_remoteFileName}"


class rtma_ru:
    def template(self):
        self.DESCRIPTION = "CONUS Rapid Update Real-Time Mesoscale Analysis (RTMA)"
        self.DETAILS = {
            "NOMADS product description": "https://www.nco.ncep.noaa.gov/pmb/products/rtma/",
            "Amazon Open Data": "https://registry.opendata.aws/noaa-rtma/",
        }
        self.PRODUCTS = {
            "anl": "Analysis",
            "err": "Analysis Forecast Error",
            "ges": "Forecasts",
        }

        raise NotImplementedError(
            "The 'rtma_ru' template is incomplete. Please edit rtma.py and make a pull request."
        )

        PATH = f"rtma2p5_ru.{self.date:%Y%m%d}/rtma2p5_ru.t{self.t:04d}z.2dvar{self.product}_ndfd.grb2"
        self.SOURCES = {
            "aws": f"https://noaa-rtma-pds.s3.amazonaws.com/{PATH}",
            "nomads": f"https://nomads.ncep.noaa.gov/pub/data/nccf/com/rtma/prod/{PATH}",
        }

        self.IDX_SUFFIX = [".grb2.idx", ".idx", ".grib.idx"]
        self.LOCALFILE = f"{self.get_remoteFileName}"


class rtma_ak:
    def template(self):
        self.DESCRIPTION = "Alaska Real-Time Mesoscale Analysis (RTMA)"
        self.DETAILS = {
            "NOMADS product description": "https://www.nco.ncep.noaa.gov/pmb/products/rtma/",
            "Amazon Open Data": "https://registry.opendata.aws/noaa-rtma/",
        }
        self.PRODUCTS = {
            "anl": "Analysis",
            "err": "Analysis Forecast Error",
            "ges": "Forecasts",
        }

        PATH = f"akrtma.{self.date:%Y%m%d}/akrtma.t{self.date:%H}z.2dvar{self.product}_ndfd_3p0.grb2"
        self.SOURCES = {
            "aws": f"https://noaa-rtma-pds.s3.amazonaws.com/{PATH}",
            "nomads": f"https://nomads.ncep.noaa.gov/pub/data/nccf/com/rtma/prod/{PATH}",
        }

        self.IDX_SUFFIX = [".grb2.idx", ".idx", ".grib.idx"]
        self.LOCALFILE = f"{self.get_remoteFileName}"


class rtma_hi:
    def template(self):
        self.DESCRIPTION = "Hawaii Real-Time Mesoscale Analysis (RTMA)"
        self.DETAILS = {
            "NOMADS product description": "https://www.nco.ncep.noaa.gov/pmb/products/rtma/",
            "Amazon Open Data": "https://registry.opendata.aws/noaa-rtma/",
        }
        self.PRODUCTS = {
            "anl": "Analysis",
            "err": "Analysis Forecast Error",
            "ges": "Forecasts",
        }

        PATH = f"hirtma.{self.date:%Y%m%d}/hirtma.t{self.date:%H}z.2dvar{self.product}_ndfd.grb2"
        self.SOURCES = {
            "aws": f"https://noaa-rtma-pds.s3.amazonaws.com/{PATH}",
            "nomads": f"https://nomads.ncep.noaa.gov/pub/data/nccf/com/rtma/prod/{PATH}",
        }

        self.IDX_SUFFIX = [".grb2.idx", ".idx", ".grib.idx"]
        self.LOCALFILE = f"{self.get_remoteFileName}"


class rtma_pr:
    def template(self):
        self.DESCRIPTION = "Puerto Rico Real-Time Mesoscale Analysis (RTMA)"
        self.DETAILS = {
            "NOMADS product description": "https://www.nco.ncep.noaa.gov/pmb/products/rtma/",
            "Amazon Open Data": "https://registry.opendata.aws/noaa-rtma/",
        }
        self.PRODUCTS = {
            "anl": "Analysis",
            "err": "Analysis Forecast Error",
            "ges": "Forecasts",
        }

        PATH = f"prrtma.{self.date:%Y%m%d}/prrtma.t{self.date:%H}z.2dvar{self.product}_ndfd.grb2"
        self.SOURCES = {
            "aws": f"https://noaa-rtma-pds.s3.amazonaws.com/{PATH}",
            "nomads": f"https://nomads.ncep.noaa.gov/pub/data/nccf/com/rtma/prod/{PATH}",
        }

        self.IDX_SUFFIX = [".grb2.idx", ".idx", ".grib.idx"]
        self.LOCALFILE = f"{self.get_remoteFileName}"


class rtma_gu:
    def template(self):
        self.DESCRIPTION = "Guam Real-Time Mesoscale Analysis (RTMA)"
        self.DETAILS = {
            "NOMADS product description": "https://www.nco.ncep.noaa.gov/pmb/products/rtma/",
            "Amazon Open Data": "https://registry.opendata.aws/noaa-rtma/",
        }
        self.PRODUCTS = {
            "anl": "Analysis",
            "err": "Analysis Forecast Error",
            "ges": "Forecasts",
        }

        PATH = f"gurtma.{self.date:%Y%m%d}/gurtma.t{self.date:%H}z.2dvar{self.product}_ndfd.grb2"
        self.SOURCES = {
            "aws": f"https://noaa-rtma-pds.s3.amazonaws.com/{PATH}",
            "nomads": f"https://nomads.ncep.noaa.gov/pub/data/nccf/com/rtma/prod/{PATH}",
        }

        self.IDX_SUFFIX = [".grb2.idx", ".idx", ".grib.idx"]
        self.LOCALFILE = f"{self.get_remoteFileName}"



## Brian Blaylock
## October 1, 2021

"""
I have no idea what I'm doing yet. First time making a test.
"""
import numpy as np


def test_sqrt():
    num = 25
    assert np.sqrt(num) == 5


def test_str():
    a = "Herbie"
    b = "Herbie"
    assert a == b



## Added by Brian Blaylock
## August 10, 2022

"""
A Herbie template for the NAM model.
"""


class nam:
    def template(self):
        self.DESCRIPTION = "North America Mesoscale - CONUS"
        self.DETAILS = {
            "NOMADS product description": "https://www.nco.ncep.noaa.gov/pmb/products/nam/",
        }
        self.PRODUCTS = {
            # TODO: Can add to this list. Look here: https://www.nco.ncep.noaa.gov/pmb/products/nam/
            "conusnest.hiresf": "CONUS 5 km",
            "firewxnest.hiresf": "Fire Weather 1.33 km CONUS/1.5 km Alaska",
            "alaskanest.hiresf": "Alaska 6 km",
            "hawaiinest.hiresf": "Hawaii 6 km",
            "priconest.hiresf": "Puerto Rico 3 km",
            "afwaca": "Central America/Caribbean",
        }
        self.SOURCES = {
            "aws": f"https://noaa-nam-pds.s3.amazonaws.com/nam.{self.date:%Y%m%d}/nam.t{self.date:%H}z.{self.product}{self.fxx:02d}.tm00.grib2",
            "nomads": f"https://nomads.ncep.noaa.gov/pub/data/nccf/com/nam/prod/nam.{self.date:%Y%m%d}/nam.t{self.date:%H}z.{self.product}{self.fxx:02d}.tm00.grib2",
        }
        self.EXPECT_IDX_FILE = "remote"
        self.LOCALFILE = f"{self.get_remoteFileName}"



## Added by Brian Blaylock
## July 26, 2021

# NOTE: The Rapid Refresh Forecast System is rapidly changing


class rrfs:
    def template(self):
        self.DESCRIPTION = "Rapid Refresh Forecast System (RRFS) Ensemble"
        self.DETAILS = {
            "aws product description": "https://registry.opendata.aws/noaa-rrfs/",
        }
        self.PRODUCTS = {
            # Below are ensemble products found in ensprod/
            "mean": "ensemble mean",
            "avrg": "ensemble products: ???",
            "eas": "ensemble products: ???",
            "ffri": "ensemble products: ???",
            "lpmm": "ensemble products: ???",
            "pmmn": "ensemble products: ???",
            "prob": "ensemble products: ???",
            # Below are member products found in mem##/
            "testbed.conus": "surface grids (one for each member)",
            "na": "native grids (one for each member)",
        }
        self.SOURCES = {
            "aws": f"https://noaa-rrfs-pds.s3.amazonaws.com/rrfs.{self.date:%Y%m%d/%H}/ensprod/rrfsce.t{self.date:%H}z.conus.{self.product}.f{self.fxx:02d}.grib2",
            "aws-mem": f"https://noaa-rrfs-pds.s3.amazonaws.com/rrfs.{self.date:%Y%m%d/%H}/mem{self.member:02d}/rrfs.t{self.date:%H}z.mem{self.member:02d}.{self.product}f{self.fxx:03d}.grib2",
        }
        self.LOCALFILE = f"mem{self.member:02d}/{self.get_remoteFileName}"



## Brian Blaylock
## April 12, 2022

"""
================
Some Herbie Help
================
"""


def _searchString_help(kind="wgrib2"):
    """
    Help/Error Message for `searchString`

    Parameters
    ----------
    kind : {"wgrib2", "eccodes"}
        There are two different utilities used to create index files and
        they create different file output.

        - **wgrib2** is the NCEP-style grib messages
        - **eccodes** is the ECMWF-style grib messages
    """

    if kind == "wgrib2":
        msg = """
Use regular expression to search for lines in the index file.
Here are some examples you can use for the wgrib2-style `searchString`

    ==================================== ==========================================================
    ``searchString=``                    GRIB messages that will be downloaded
    ==================================== ==========================================================
    ":TMP:2 m"                           Temperature at 2 m.
    ":TMP:"                              Temperature fields at all levels.
    ":UGRD:\d+ mb"                       U Wind at all pressure levels.
    ":500 mb:"                           All variables on the 500 mb level.
    ":APCP:"                             All accumulated precipitation fields.
    ":APCP:surface:0-[1-9]*"             Accumulated precip since initialization time
    ":APCP:.*:(?:0-1|[1-9]\d*-\d+) hour" Accumulated precip over last hour
    ":UGRD:10 m"                         U wind component at 10 meters.
    ":[U|V]GRD:[1,8]0 m"                 U and V wind component at 10 and 80 m.
    ":[U|V]GRD:"                         U and V wind component at all levels.
    ":.GRD:"                             (Same as above)
    ":[U|V]GRD:\d+ hybrid"               U and V wind components at all hybrid levels
    ":[U|V]GRD:\d+ mb"                   U and V wind components at all pressure levels
    ":(?:TMP|DPT):"                      Temperature and Dew Point for all levels .
    ":(?:TMP|DPT|RH):"                   TMP, DPT, and Relative Humidity for all levels.
    ":REFC:"                             Composite Reflectivity
    ":surface:"                          All variables at the surface.
    "^TMP:2 m.*fcst$"                    Beginning of string (^), end of string ($) wildcard (.*)
    ==================================== ==========================================================

If you need help with regular expression, search the web or look at
this cheatsheet: https://www.petefreitag.com/cheatsheets/regex/.
"""

    elif kind == "eccodes":
        msg = """
Use regular expression to search for lines in the index file.
Here are some examples you can use for the ecCodes-style `searchString`

Look at the ECMWF GRIB Parameter Database
https://apps.ecmwf.int/codes/grib/param-db

======================== ==============================================
searchString (oper/enso) Messages that will be downloaded
======================== ==============================================
":2t:"                   2-m temperature
":10u:"                  10-m u wind vector
":10v:"                  10-m v wind vector
":10(u|v):               **10m u and 10m v wind**
":d:"                    Divergence (all levels)
":gh:"                   geopotential height (all levels)
":gh:500"                geopotential height only at 500 hPa
":st:"                   soil temperature
":tp:"                   total precipitation
":msl:"                  mean sea level pressure
":q:"                    Specific Humidity
":r:"                    relative humidity
":ro:"                   Runn-off
":skt:"                  skin temperature
":sp:"                   surface pressure
":t:"                    temperature
":tcwv:"                 Total column vertically integrated water vapor
":vo:"                   Relative vorticity
":v:"                    v wind vector
":u:"                    u wind vector
":(t|u|v|r):"            Temp, u/v wind, RH (all levels)
":500:"                  All variables on the 500 hPa level

======================== ==============================================
searchString (wave/waef) Messages that will be downloaded
======================== ==============================================
":swh:"                  Significant height of wind waves + swell
":mwp:"                  Mean wave period
":mwd:"                  Mean wave direction
":pp1d:"                 Peak wave period
":mp2:"                  Mean zero-crossing wave period

If you need help with regular expression, search the web or look at
this cheatsheet: https://www.petefreitag.com/cheatsheets/regex/.
"""

    return msg



"""
Tests for Herbie xarray accessors
"""

from herbie import Herbie
from shapely.geometry import MultiPoint
import pandas as pd


def test_crs():
    H = Herbie(
        "2022-12-13 12:00",
        model="hrrr",
        product="sfc",
    )
    ds = H.xarray("TMP:2 m")
    crs = ds.herbie.crs
    assert crs


def test_nearest_points():
    ds = Herbie("2022-12-13 12:00", model="hrrr", product="sfc").xarray("TMP:2 m")

    p3 = [(-110, 50), (-100, 40), (-105, 35)]
    n3 = ["AAA", "BBB", "CCC"]

    test_points = [
        (-100, 40),
        [-100, 40],
        p3,
        # MultiPoint(p3), #Only test this if we pin shapely>=2.0
        pd.DataFrame(p3, columns=["longitude", "latitude"], index=n3),
    ]

    for p in test_points:
        ds1 = ds.herbie.nearest_points(p)
        assert len(ds1.t2m)


def test_polygon():
    H = Herbie(
        "2022-12-13 12:00",
        model="hrrr",
        product="sfc",
    )
    ds = H.xarray("TMP:2 m")
    polygons = ds.herbie.polygon
    assert len(polygons) == 2



## Brian Blaylock
## October 13, 2021

"""
Tests for downloading ECMWF model
"""
from datetime import datetime, timedelta

from herbie import Herbie

now = datetime.now()
yesterday = datetime(now.year, now.month, now.day) - timedelta(days=1)
today_str = yesterday.strftime("%Y-%m-%d %H:%M")
save_dir = "$TMPDIR/Herbie-Tests/"


def test_ecmwf():
    H = Herbie(
        yesterday,
        model="ecmwf",
        product="oper",
        save_dir=save_dir,
    )

    # Test full file download
    H.download()
    assert H.get_localFilePath().exists()

    # Test partial file download
    # temperature at all levels
    H.download(":t:")
    assert H.get_localFilePath(":t:").exists()

    # Test partial file xarray
    H.xarray(":10(?:u|v):", remove_grib=False)
    assert H.get_localFilePath(":10(?:u|v):").exists()
